from ._browser import Browser, BrowserInitOptions
from ._utils.data_packet import DataPacketProcessor
from .launcher import Launcher

__all__ = ['Launcher', 'Browser', 'BrowserInitOptions', 'DataPacketProcessor']



"""
Copyright (c) 2025-now Martian Bugs All rights reserved.
Build Date: 2025-05-19
Author: Martian Bugs
Description: 系统工具类, 例如文件操作、路径操作等
"""

from os import path, remove
from typing import Any

from openpyxl import load_workbook


class OsTools:
    @staticmethod
    def file_remove(file_path: str):
        """
        删除文件

        Args:
            file_path: 文件路径
        Returns:
            是否删除成功, 如果文件不存在或不是文件, 则返回False
        """

        if not path.exists(file_path) or not path.isfile(file_path):
            return False

        remove(file_path)
        return True

    @staticmethod
    def xlsx_read(file_path: str, sheet_name: str = None, titles: list[str] = None):
        """
        读取 xlsx 文件内容

        Args:
            file_path: xlsx 文件路径
            sheet_name: 工作表名称，默认激活的工作表
            titles: 标题列表，如果不指定，则读取第一行作为标题
        Returns:
            xlsx 内容列表
        """

        wb = load_workbook(file_path)
        sheet = (
            wb.active
            if not sheet_name or not isinstance(sheet_name, str)
            else wb[sheet_name]
        )

        titles_dict = {}
        title_row = next(sheet.rows)
        if not titles or not isinstance(titles, list):
            for ci, col in enumerate(title_row):
                titles_dict[col.value] = ci
        else:
            titles_dict = {k: -1 for k in titles}
            for ci, col in enumerate(title_row):
                if col.value in titles_dict:
                    titles_dict[col.value] = ci

        records: list[dict[str, Any]] = []
        for row in sheet.iter_rows(min_row=2):
            record = {k: v for k, ci in titles_dict.items() for v in [row[ci].value]}
            records.append(record)

        return records



"""
Copyright (c) 2025-now Martian Bugs All rights reserved.
Build Date: 2025-02-16
Author: Martian Bugs
Description: 浏览器自动化启动器
"""

from ._browser import Browser, BrowserInitOptions


class Launcher:
    def init_browser(self, init_options: BrowserInitOptions):
        """
        初始化浏览器

        Args:
            init_options: BrowserInitOptions 对象
        Returns:
            浏览器对象
        """

        return Browser(init_options)



from dataclasses import dataclass
from os import makedirs, path
from random import randint

from DrissionPage import Chromium, ChromiumOptions


@dataclass
class BrowserWindowLoc:
    width: int = None
    height: int = None
    x: int = None
    y: int = None


class BrowserInitOptions:
    def __init__(self):
        self._browser_path = None
        self._user_data_dirpath = None
        self._port = None

        self._browser_window_loc = None

    def set_basic_options(
        self,
        browser_path: str = None,
        user_data_dirpath: str = None,
        port=9333,
    ):
        """
        设置基础参数

        Args:
            browser_path: 浏览器可执行文件路径
            user_data_dirpath: 用户数据目录路径
            port: 端口号
            auto_close: 是否自动关闭浏览器
        """

        self._browser_path = browser_path
        self._user_data_dirpath = user_data_dirpath
        self._port = port

        return self

    def set_random_port(self, begin=9333, end=999):
        """
        设置随机端口号

        Args:
            begin: 随机端口号起始值
            end: 随机端口号结束值
        """

        self._port = randint(begin, end)

        return self

    def set_window_loc(self, width=1400, height=800, x=20, y=20):
        """
        设置浏览器窗口位置

        Args:
            width: 窗口宽度
            height: 窗口高度
            x: 窗口左上角x坐标
            y: 窗口左上角y坐标
        """

        self._browser_window_loc = BrowserWindowLoc(width, height, x, y)

        return self


class Browser:
    def __init__(self, options: BrowserInitOptions):
        self._options = options
        self._chromium = self.__connect()

    def __connect(self):
        """连接浏览器"""

        option_paths = {
            'local_port': self._options._port,
        }

        browser_path = self._options._browser_path
        if browser_path and isinstance(browser_path, str):
            if not path.exists(browser_path):
                raise FileNotFoundError(f'浏览器执行文件 [{browser_path}] 不存在')
            option_paths['browser_path'] = browser_path

        user_data_dirpath = self._options._user_data_dirpath
        if user_data_dirpath and isinstance(user_data_dirpath, str):
            if not path.exists(user_data_dirpath) or not path.isdir(user_data_dirpath):
                makedirs(user_data_dirpath, exist_ok=True)

            option_paths['user_data_path'] = user_data_dirpath

        option = ChromiumOptions(read_file=False)
        option.set_paths(**option_paths)
        option.set_argument('--disable-background-networking')
        option.set_argument('--hide-crash-restore-bubble')
        option.set_pref('credentials_enable_service', False)

        chromium = Chromium(addr_or_opts=option)
        chromium.set.load_mode.none()

        if isinstance(self._options._browser_window_loc, BrowserWindowLoc):
            latest_tab = chromium.latest_tab
            latest_tab.set.window.size(
                width=self._options._browser_window_loc.width,
                height=self._options._browser_window_loc.height,
            )
            latest_tab.set.window.location(
                x=self._options._browser_window_loc.x,
                y=self._options._browser_window_loc.y,
            )

        return chromium

    @property
    def port(self):
        return self._options._port

    @property
    def chromium(self):
        return self._chromium

    def quit(self):
        """退出浏览器"""

        if not isinstance(self._chromium, Chromium):
            return

        self._chromium.quit(force=True)



"""
Copyright (c) 2025-now Martian Bugs All rights reserved.
Build Date: 2025-05-19
Author: Martian Bugs
Description: 字典相关工具库
"""

from copy import deepcopy


class DictUtils:
    @staticmethod
    def dict_mapping(data: dict, dict_table: dict[str, str]):
        """
        字典表字段映射

        Args:
            data: 待映射的字典
            dict_table: 字典表
        """

        result = {}
        for text, key in dict_table.items():
            result[text] = data.get(key)

        return result

    @staticmethod
    def dict_format__float(
        data: dict,
        fields: list[str] = None,
        precision: int = 2,
        exclude_fields: list[str] = None,
    ):
        """
        将字典数据中的指定字段格式化为 float 类型

        Args:
            data: 待格式化的字典
            fields: 需要格式化的字段列表, 如果为 None, 则表示所有字段
            precision: 保留小数位数, 默认 2 位
            exclude_fields: 排除的字段列表
        Returns:
            格式化后的字典
        """

        _fields = fields if fields and isinstance(fields, list) else data.keys()
        _exclude_fields = (
            exclude_fields
            if exclude_fields and isinstance(exclude_fields, list)
            else []
        )

        _data = deepcopy(data)
        for field in _fields:
            if field not in _data or field in _exclude_fields:
                continue

            value = _data[field]
            if not isinstance(value, (int, float)):
                continue

            _data[field] = value / 10**precision

        return _data

    @staticmethod
    def dict_format__round(
        data: dict,
        fields: list[str] = None,
        precision: int = 2,
        exclude_fields: list[str] = None,
    ):
        """
        将字典数据中的指定字段作四舍五入处理

        Args:
            data: 待格式化的字典
            fields: 需要格式化的字段列表, 如果为 None, 则表示所有字段
            precision: 保留小数位数, 默认 2 位
            exclude_fields: 排除的字段列表
        Returns:
            格式化后的字典
        """

        _fields = fields if fields and isinstance(fields, list) else data.keys()
        _exclude_fields = (
            exclude_fields
            if exclude_fields and isinstance(exclude_fields, list)
            else []
        )

        _data = deepcopy(data)
        for field in _fields:
            if field not in _data or field in _exclude_fields:
                continue

            value = _data[field]
            if not isinstance(value, float):
                continue

            _data[field] = round(value, precision)

        return _data

    @staticmethod
    def dict_format__ratio(
        data: dict,
        fields: list[str] = None,
        ratio: int = 2,
        exclude_fields: list[str] = None,
    ):
        """
        将字典数据中的指定字段转为比率, 例如百分比/千分比等

        Args:
            data: 待格式化的字典
            fields: 需要格式化的字段列表, 如果为 None, 则表示所有字段
            ratio: 比率, 默认 2 及百分比
            exclude_fields: 排除的字段列表
        Returns:
            格式化后的字典
        """

        _fields = fields if fields and isinstance(fields, list) else data.keys()
        _exclude_fields = (
            exclude_fields
            if exclude_fields and isinstance(exclude_fields, list)
            else []
        )

        _data = deepcopy(data)
        for field in _fields:
            if field not in _data or field in _exclude_fields:
                continue

            value = _data[field]
            if not isinstance(value, (int, float)):
                continue

            _data[field] = value * (10**ratio)

        return _data

    @staticmethod
    def dict_format__strip(
        data: dict,
        fields: list[str] = None,
        prefix: list[str] = None,
        suffix: list[str] = None,
        exclude_fields: list[str] = None,
    ):
        """
        格式化字典数据中的指定字段, 去除前后空格及指定前后缀

        Args:
            data: 待格式化的字典
            fields: 需要格式化的字段列表, 如果为 None, 则表示所有字段
            prefix: 需要去除的前缀列表
            suffix: 需要去除的后缀列表
            exclude_fields: 排除的字段列表
        Returns:
            格式化后的字典
        """

        _fields = fields if fields and isinstance(fields, list) else data.keys()
        _exclude_fields = (
            exclude_fields
            if exclude_fields and isinstance(exclude_fields, list)
            else []
        )

        _data = deepcopy(data)
        for field in _fields:
            if field not in _data or field in _exclude_fields:
                continue

            value = _data[field]
            if not isinstance(value, str):
                continue

            value = value.strip()
            if prefix and isinstance(prefix, list):
                for c in prefix:
                    value = value.lstrip(c)

            if suffix and isinstance(suffix, list):
                for c in suffix:
                    value = value.rstrip(c)

            _data[field] = value

        return _data

    @staticmethod
    def dict_format__number(
        data: dict, fields: list[str] = None, exclude_fields: list[str] = None
    ):
        """
        格式化字典数据中的指定字段, 将字符串转为数字

        Args:
            data: 待格式化的字典
            fields: 需要格式化的字段列表, 如果为 None, 则表示所有字段
            exclude_fields: 排除的字段列表
        Returns:
            格式化后的字典
        """

        _fields = fields if fields and isinstance(fields, list) else data.keys()
        _exclude_fields = (
            exclude_fields
            if exclude_fields and isinstance(exclude_fields, list)
            else []
        )

        _data = deepcopy(data)
        for field in _fields:
            if field not in _data or field in _exclude_fields:
                continue

            value = _data[field]
            if not isinstance(value, str):
                continue

            try:
                value = value.replace(',', '')
                value = float(value) if '.' in value else int(value)
            except ValueError:
                continue

            _data[field] = value

        return _data



"""
Copyright (c) 2025-now Martian Bugs All rights reserved.
Build Date: 2025-05-19
Author: Martian Bugs
Description: web 相关的工具库
"""

from urllib.parse import quote, unquote, urlencode, urlparse


class WebTools:
    @staticmethod
    def url_same(a: str, b: str):
        """
        检查两个 url 是否域名及路径是否一致

        Args:
            a: 第一个 url
            b: 第二个 url
        Returns:
            是否一致
        """

        a_result = urlparse(a)
        b_result = urlparse(b)

        is_same = a_result.netloc == b_result.netloc and a_result.path == b_result.path

        return is_same

    @staticmethod
    def url_append_params(url: str, params: dict):
        """
        给 url 添加参数

        Args:
            url: 原始 url
            params: 参数字典
        Returns:
            带参数的 url
        """

        _url = url.rstrip('?')
        params_str = urlencode(params)
        return f'{_url}?{params_str}'

    @staticmethod
    def url_decode(url: str):
        """
        url 链接解码

        Args:
            url: 待解码的 url
        Returns:
            解码后的 url
        """

        return unquote(url)

    @staticmethod
    def url_encode(url: str):
        """
        url 链接编码

        Ags:
            url: 待编码的 url
        Returns:
            编码后的 url
        """

        return quote(url)



"""
Copyright (c) 2025-now Martian Bugs All rights reserved.
Build Date: 2025-05-19
Author: Martian Bugs
Description: 日期时间工具库
"""

from datetime import datetime, timedelta, timezone


class DateTimeTools:
    @staticmethod
    def date_yesterday(pattern='%Y-%m-%d'):
        """
        获取前一天的日期

        Args:
            pattern: 日期格式
        """

        return DateTimeTools.date_calculate(days=1, pattern=pattern)

    @staticmethod
    def date_calculate(days: int, pattern='%Y-%m-%d'):
        """
        日期计算

        Args:
            days: 日期偏移量, 负数表示向后推
            pattern: 日期格式
        """

        return (datetime.now() - timedelta(days=days)).strftime(pattern)

    @staticmethod
    def date_diff_days(a: str, b: str, pattern='%Y-%m-%d'):
        """
        计算两个日期间隔的天数
        - 正数表示 a 日期在 b 日期之后

        Args:
            a: 日期字符串
            b: 日期字符串
        """

        a_dt = datetime.strptime(a, pattern)
        b_dt = datetime.strptime(b, pattern)

        return (a_dt - b_dt).days

    @staticmethod
    def date_to_timestamp(date: str, pattern='%Y-%m-%d'):
        """
        将日期转为 10 位时间戳

        Args:
            date: 日期字符串
            pattern: 日期格式
        """

        dt = datetime.strptime(date, pattern)
        dt = dt.replace(tzinfo=timezone(timedelta(hours=8)))
        return int(dt.timestamp())

    @staticmethod
    def datetime_to_timestamp(date_time: str, pattern='%Y-%m-%d %H:%M:%S'):
        """
        将时间时间转为 13 位时间戳

        Args:
            date_time: 日期时间字符串
            pattern: 日期时间格式
        """

        dt = datetime.strptime(date_time, pattern)
        dt = dt.replace(tzinfo=timezone(timedelta(hours=8)))
        return int(dt.timestamp() * 1000)

    @staticmethod
    def date_range(begin_date: str, end_date: str, pattern='%Y-%m-%d'):
        """
        根据起始日期和结束日期生成日期列表

        Args:
            begin_date: 起始日期字符串
            end_date: 结束日期字符串
            pattern: 日期格式
        Returns:
            日期列表
        """

        begin_dt, end_dt = [
            datetime.strptime(date, pattern) for date in (begin_date, end_date)
        ]
        diff = (end_dt - begin_dt).days
        date_list = [
            (begin_dt + timedelta(days=i)).strftime(pattern) for i in range(diff + 1)
        ]

        return date_list

    @staticmethod
    def timestamp_to_str(timestamp: int | str, pattern='%Y-%m-%d %H:%M:%S'):
        """
        将时间戳转为日期时间字符串

        Args:
            timestamp: 时间戳
            pattern: 日期时间格式
        Returns:
            日期时间字符串
        """

        return datetime.fromtimestamp(int(timestamp)).strftime(pattern)

    @staticmethod
    def seconds_to_time(seconds: int):
        """
        秒数转为时间字符串

        Args:
            seconds: 秒数
        Returns:
            时间字符串, 格式为 'xx:xx'
        """

        m, s = divmod(seconds, 60)
        _, m = divmod(m, 60)
        return f'{m:02d}:{s:02d}'



"""
Copyright (c) 2025-now Martian Bugs All rights reserved.
Build Date: 2025-04-02
Author: Martian Bugs
Description: 数据包处理器
"""

from typing import Any

from DrissionPage._units.listener import DataPacket


class DataPacketProcessor:
    def __init__(self, packet: DataPacket):
        self._packet = packet
        self._resp = self._packet.response.body

    def filter(self, fields: str | list[str] | list[tuple[str, str]]):
        """
        筛选数据包字段数据

        Args:
            fields: 以 `.` 分隔的字段层级结构, 支持列表 key[index] 形式.\n
                如果需要定义别名, 则以 tuple 形式传入 (key, alias).\n
                默认会进行校验, 如果不需要校验则可以在链路最前面加上 `?` 符号,\n
                如果只是某一个字段不校验, 则在字段后面加上 `?` 符号\n
                例如: `['?resule.version', 'resule.datas[0].name?', ('resule.extra[1].name', 'extra_name')]`
        Returns:
            提取的字段数据
        """

        if not isinstance(self._resp, dict):
            raise TypeError('数据包非预期的 dict 类型')

        field_list = fields if isinstance(fields, list) else [fields]

        result: dict[str, Any] = {}
        for chain in field_list:
            if not isinstance(chain, (str, tuple)):
                continue

            chain_keys = (chain[0] if isinstance(chain, tuple) else chain).split('.')
            need_verify = not chain_keys[0].startswith('?')
            if not need_verify:
                chain_keys[0] = chain_keys[0].lstrip('?')

            data_key__temp = None
            data_temp = self._resp

            for i, field in enumerate(chain_keys, 1):
                field_need_verify = not field.endswith('?')
                data_key__temp = field = (
                    field.rstrip('?') if not field_need_verify else field
                )
                prev_field_chain = '.'.join(chain_keys[: i - 1]).rstrip('?')
                prev_field_chain = prev_field_chain if prev_field_chain else 'root'

                if '[' in field and ']' in field:
                    # 处理列表形式的 key[index]

                    field, list_index_str = field.split('[')
                    if field not in data_temp:
                        if need_verify and field_need_verify:
                            raise KeyError(f'{prev_field_chain} 中未找到 {field} 字段')
                        else:
                            data_key__temp = None
                            break

                    curr_field_chain = '.'.join([prev_field_chain, field])
                    data_temp = data_temp.get(field)
                    if not isinstance(data_temp, list):
                        if need_verify and field_need_verify:
                            raise TypeError(f'{curr_field_chain} 非预期的 list 类型')
                        else:
                            data_key__temp = None
                            break

                    list_index = int(list_index_str.rstrip(']'))
                    if list_index >= (data_temp__count := len(data_temp)):
                        if need_verify and field_need_verify:
                            raise IndexError(
                                f'{curr_field_chain} 索引超出范围, 长度为 {data_temp__count}'
                            )
                        else:
                            data_key__temp = None
                            break

                    data_key__temp = field
                    data_temp = data_temp[list_index]
                    continue

                if not isinstance(data_temp, dict):
                    if need_verify and field_need_verify:
                        raise TypeError(f'{prev_field_chain} 非预期的 dict 类型')
                    else:
                        data_key__temp = None
                        continue

                if field not in data_temp:
                    if need_verify and field_need_verify:
                        raise KeyError(f'{prev_field_chain} 中未找到 {field} 字段')
                    else:
                        data_key__temp = None
                        continue

                data_temp = data_temp.get(data_key__temp)

            if data_key__temp is None:
                continue

            result[chain[1] if isinstance(chain, tuple) else data_key__temp] = data_temp

        return result
